@@ -4,6 +4,10 @@
 
 #![warn(missing_docs)]
 
+use serde::{de, Deserialize};
+use serde_json::{value::RawValue as RawJsonValue, Value as JsonValue};
+
+pub mod base64;
 mod buf;
 pub mod can_be_empty;
 mod canonical_json;
@@ -17,21 +21,27 @@ mod strings;
 pub mod test;
 pub mod urlencoded;
 
-pub use buf::{json_to_buf, slice_to_buf};
-pub use can_be_empty::{is_empty, CanBeEmpty};
-pub use canonical_json::{
-    to_canonical_value, try_from_json_map,
-    value::{CanonicalJsonValue, Object as CanonicalJsonObject},
-    Error as CanonicalJsonError,
-};
-pub use cow::deserialize_cow_str;
-pub use empty::vec_as_map_of_empty;
-pub use raw::Raw;
-pub use strings::{
-    btreemap_int_or_string_to_int_values, empty_string_as_none, int_or_string_to_int,
-    none_as_empty_string,
+pub use self::{
+    base64::{Base64, Base64DecodeError},
+    buf::{json_to_buf, slice_to_buf},
+    can_be_empty::{is_empty, CanBeEmpty},
+    canonical_json::{
+        to_canonical_value, try_from_json_map,
+        value::{CanonicalJsonValue, Object as CanonicalJsonObject},
+        Error as CanonicalJsonError,
+    },
+    cow::deserialize_cow_str,
+    empty::vec_as_map_of_empty,
+    raw::Raw,
+    strings::{
+        btreemap_deserialize_v1_powerlevel_values, deserialize_v1_powerlevel, empty_string_as_none,
+        none_as_empty_string,
+    },
 };
 
+/// The inner type of [`JsonValue::Object`].
+pub type JsonObject = serde_json::Map<String, JsonValue>;
+
 /// Check whether a value is equal to its default value.
 pub fn is_default<T: Default + PartialEq>(val: &T) -> bool {
     *val == T::default()
@@ -46,12 +56,21 @@ pub fn default_true() -> bool {
 
 /// Simply dereferences the given bool.
 ///
-/// Useful for `#[serde(skip_serializing_if = ...)]`
+/// Useful for `#[serde(skip_serializing_if = ...)]`.
 #[allow(clippy::trivially_copy_pass_by_ref)]
 pub fn is_true(b: &bool) -> bool {
     *b
 }
 
+/// Helper function for `serde_json::value::RawValue` deserialization.
+pub fn from_raw_json_value<'a, T, E>(val: &'a RawJsonValue) -> Result<T, E>
+where
+    T: Deserialize<'a>,
+    E: de::Error,
+{
+    serde_json::from_str(val.get()).map_err(E::custom)
+}
+
 /// A type that can be sent to another party that understands the matrix protocol.
 ///
 /// If any of the fields of `Self` don't implement serde's `Deserialize`, you can derive this trait
